# -*- coding: utf-8 -*-

from math import pi, sin, cos
from direct.showbase.ShowBase import ShowBase
from direct.task import Task
from direct.actor import Actor
from pandac.PandaModules import *
from direct.interval.IntervalGlobal import *
import sys, os
from direct.gui.DirectGui import *

loadPrcFileData('', 'window-title Tanki')
loadPrcFileData('', 'fullscreen 0')
#loadPrcFileData('', 'red-blue-stereo 1')
#loadPrcFileData('', 'win-size 1440 900')
#loadPrcFileData('', 'win-size 1024 600')
#loadPrcFileData('', 'window-inverted 1')


from direct.gui.OnscreenText import OnscreenText

class MyApp(ShowBase):
	def __init__(self):
		ShowBase.__init__(self)	
		
		# общее освещение
		self.ambientLight = render.attachNewNode( AmbientLight( "ambientLight" ) )
		self.ambientLight.node().setColor( Vec4(0.6,0.6,0.6,1) )
		# точечный источник
		self.PointLight = camera.attachNewNode( PointLight( "PointLight" ) )
		self.PointLight.node().setColor( Vec4(1,1,1,1) )
		self.PointLight.node().setAttenuation( Vec3(.1,0.04,0.0) ) 
		# Говорим, что нужно освещать рендер
		render.setLight( self.ambientLight )
		render.setLight( self.PointLight )
		
		# текстовка хелпа
		#########################################################
		self.font = loader.loadFont('arial.ttf')
		self.title = self.addTitle(u"BETA ТАНЧИКИ :)")
		self.inst1 = self.addInstructions(0.95, u"[ESC]: Выход")
		self.inst2 = self.addInstructions(0.90, u"[Left Arrow]: Поворот танка в лево")
		self.inst3 = self.addInstructions(0.85, u"[Right Arrow]: Поворот танка в право")
		self.inst4 = self.addInstructions(0.80, u"[Up Arrow]: Вперед")
		self.inst5 = self.addInstructions(0.75, u"[Down Arrow]: Назад")
		self.inst6 = self.addInstructions(0.70, u"[A]: Поворот башни влево")
		self.inst7 = self.addInstructions(0.65, u"[S]: Поворот башни вправо")
		self.inst8 = self.addInstructions(0.60, u"[A]: Поворот камеры влево")
		self.inst9 = self.addInstructions(0.55, u"[S]: Поворот камеры вправо")
		self.inst10 = self.addInstructions(0.50, u"[PgUp]: Поднять камеру")
		self.inst11 = self.addInstructions(0.45, u"[PgDown]: Опустить камеру")
		self.inst12 = self.addInstructions(0.40, u"[Space]: Выстрел")
		#############################################################
		
		# ключи для управления через клаву
		self.keyMap = {	"norma":0,"left":0, "right":0, "forward":0, "back":0, 
										"bashnya-left":0, "bashnya-right":0, "cam-up":0,
										"cam-down":0, "fire":0, "cam-left":0, "cam-right":0}
		#############################################################
		
		# грузим карту
		self.environ = self.loader.loadModel("./res/geometry/mapa22")
		self.environ.reparentTo(self.render)
		self.environ.setScale(0.0025, 0.0025, 0.0025)
		self.environ.setPos(0,0,0)

		# грузим танк
		self.tank = self.loader.loadModel("res/actors/tank")
		self.tank.reparentTo(self.render)
		self.tank.setScale(0.0003,0.0003,0.0003)
		tX,tY,tZ,tH,tP,tR=0,0,0.932,0,0,0
		self.tank.setPos(tX,tY,tZ)
		self.tank.setHpr(tH,tP,tR)
		# угол поворота корпуса танка
		self.a = self.tank.getH()
		
		# даем некоторые теги для плавной обработки движения танка по поверхности
		self.tank.setTag('angleHold', str(self.tank.getH()))
		self.tank.setTag('anglePold', str(self.tank.getP()))
		self.tank.setTag('angleRold', str(self.tank.getR()))
		self.tank.setTag('Zold', str(self.tank.getZ()))
		
		# разносим танк на детали
		self.stvol = self.tank.find('**/**/stvol')
		self.bashnya = self.tank.find('**/**/bashnya')

		# грузим сферу HPRtank
		self.hprTank = self.loader.loadModel("res/geometry/hpr")
		self.hprTank.reparentTo(self.render)
		self.hprTank.setScale(0.0003,0.0003,0.0003)
		self.hprTank.setPos(0,0,2)
		# грузим сферу HPRpole
		self.hprPole = self.loader.loadModel("res/geometry/hpr")
		self.hprPole.reparentTo(self.render)
		self.hprPole.setScale(0.0003,0.0003,0.0003)
		self.hprPole.setPos(0,0,2)
		###########
		self.textObject = OnscreenText(text = "", pos = (0, 0.7), scale = 0.07, font = self.font, fg=(1,1,1,1), shadow=(0,0,0,1))
########################################################################################################
		# связываем клавиши с действиями
		self.accept("escape", sys.exit)
		self.accept("arrow_left", self.setKey, ["left",1])
		self.accept("arrow_right", self.setKey, ["right",1])
		self.accept("arrow_up", self.setKey, ["forward",1])
		self.accept("arrow_down", self.setKey, ["back",1])
		self.accept("a", self.setKey, ["bashnya-left",1])
		self.accept("s", self.setKey, ["bashnya-right",1])
		
		self.accept("q", self.setKey, ["cam-left",1])
		self.accept("w", self.setKey, ["cam-right",1])
		
		self.accept("space", self.setKey, ["fire",1])
		self.accept("space-up", self.setKey, ["fire",0])
		
		self.accept("n", self.setKey, ["norma",1])
		self.accept("n-up", self.setKey, ["norma",0])
		
		self.accept("page_up", self.setKey, ["cam-up",1])
		self.accept("page_up-up", self.setKey, ["cam-up",0])
		
		self.accept("page_down", self.setKey, ["cam-down",1])
		self.accept("page_down-up", self.setKey, ["cam-down",0])
		
		self.accept("arrow_left-up", self.setKey, ["left",0])
		self.accept("arrow_right-up", self.setKey, ["right",0])
		self.accept("arrow_up-up", self.setKey, ["forward",0])
		self.accept("arrow_down-up", self.setKey, ["back",0])
		self.accept("a-up", self.setKey, ["bashnya-left",0])
		self.accept("s-up", self.setKey, ["bashnya-right",0])
		
		self.accept("q-up", self.setKey, ["cam-left",0])
		self.accept("w-up", self.setKey, ["cam-right",0])
		
		# включаем обработчик
		taskMgr.add(self.move,"moveTask")
		
		# отключаем стандартное управление мышой
		base.disableMouse()
		
		# определяем водителя для камеры
		self.boxCam = render.attachNewNode('boxCam') 
		self.boxCam.setPos(self.tank.getPos())
		self.boxCam.setZ(1)
		self.boxCam.setHpr(self.tank.getHpr())
		
		# устанавлеваем начальное положение камеры
		base.camera.setPos(self.boxCam.getX(), self.boxCam.getY()-5, 1)
		base.camera.reparentTo(self.boxCam)
		base.camera.lookAt(self.boxCam)
		
		###################################################################################################
		#                   К О Л Л И З И И
		self.cTrav = CollisionTraverser()
		
		# луч для коллиции с полом и нахождения нормали к поверхности
		# 1
		self.tankGroundRay1 = CollisionRay(0,0,3000,0,0,-1)
		self.tankGroundCol1 = CollisionNode('tankRay')
		self.tankGroundCol1.addSolid(self.tankGroundRay1)
		self.tankGroundCol1.setFromCollideMask(BitMask32.bit(1))
		self.tankGroundCol1.setIntoCollideMask(BitMask32.allOff())
		self.tankGroundColNp1 = self.tank.attachNewNode(self.tankGroundCol1)
		self.tankGroundHandler1 = CollisionHandlerQueue()
		self.cTrav.addCollider(self.tankGroundColNp1, self.tankGroundHandler1)
		self.tankGroundColNp1.show()
		
		# 2
		self.tankGroundRay2 = CollisionRay(0,-2300,3000,0,0,-1)
		self.tankGroundCol2 = CollisionNode('tankRay')
		self.tankGroundCol2.addSolid(self.tankGroundRay2)
		self.tankGroundCol2.setFromCollideMask(BitMask32.bit(1))
		self.tankGroundCol2.setIntoCollideMask(BitMask32.allOff())
		self.tankGroundColNp2 = self.tank.attachNewNode(self.tankGroundCol2)
		self.tankGroundHandler2 = CollisionHandlerQueue()
		self.cTrav.addCollider(self.tankGroundColNp2, self.tankGroundHandler2)
		self.tankGroundColNp2.show()
		
		# четыре шара по углам танка для коллиции с препядствиями
		# 1
		self.EnvDetector1=CollisionSphere(-700, 1900, 1900, 1000)
		self.EnvCol1 = CollisionNode('colEnv_1')
		self.EnvCol1.addSolid(self.EnvDetector1)
		self.EnvCol1.setFromCollideMask(BitMask32.bit(2))
		self.EnvCol1.setIntoCollideMask(BitMask32.allOff())
		self.EnvColNp1 = self.tank.attachNewNode(self.EnvCol1)
		#self.EnvColNp1.show()
		self.pusher1 = CollisionHandlerPusher()
		self.pusher1.addCollider(self.EnvColNp1, self.tank)
		self.cTrav.addCollider(self.EnvColNp1, self.pusher1)
		
		# 2
		self.EnvDetector2=CollisionSphere(700, 1900, 1900, 1000)
		self.EnvCol2 = CollisionNode('colEnv_2')
		self.EnvCol2.addSolid(self.EnvDetector2)
		self.EnvCol2.setFromCollideMask(BitMask32.bit(2))
		self.EnvCol2.setIntoCollideMask(BitMask32.allOff())
		self.EnvColNp2 = self.tank.attachNewNode(self.EnvCol2)
		#self.EnvColNp2.show()
		self.pusher2 = CollisionHandlerPusher()
		self.pusher2.addCollider(self.EnvColNp2, self.tank)
		self.cTrav.addCollider(self.EnvColNp2, self.pusher2)
		
		# 3
		self.EnvDetector3=CollisionSphere(-700, -1900, 1900, 1000)
		self.EnvCol3 = CollisionNode('colEnv_3')
		self.EnvCol3.addSolid(self.EnvDetector3)
		self.EnvCol3.setFromCollideMask(BitMask32.bit(2))
		self.EnvCol3.setIntoCollideMask(BitMask32.allOff())
		self.EnvColNp3 = self.tank.attachNewNode(self.EnvCol3)
		#self.EnvColNp3.show()
		self.pusher3 = CollisionHandlerPusher()
		self.pusher3.addCollider(self.EnvColNp3, self.tank)
		self.cTrav.addCollider(self.EnvColNp3, self.pusher3)
		
		# 4
		self.EnvDetector4=CollisionSphere(700, -1900, 1900, 1000)
		self.EnvCol4 = CollisionNode('colEnv_4')
		self.EnvCol4.addSolid(self.EnvDetector4)
		self.EnvCol4.setFromCollideMask(BitMask32.bit(2))
		self.EnvCol4.setIntoCollideMask(BitMask32.allOff())
		self.EnvColNp4 = self.tank.attachNewNode(self.EnvCol4)
		#self.EnvColNp4.show()
		self.pusher4 = CollisionHandlerPusher()
		self.pusher4.addCollider(self.EnvColNp4, self.tank)
		self.cTrav.addCollider(self.EnvColNp4, self.pusher4)
		
		
		#
		#self.cTrav.showCollisions(render)
		###################################################################################################
		
		#self.tank.hide()
#################################################################################################
	# обработчик нажатия клавиши
	def setKey(self, key, value):
		self.keyMap[key] = value
	
	# обработчик движения танка камеры ствола башни и прочих камер
	def move(self, task):
		elapsed = globalClock.getDt()
		
		# поворот камеры по горизонту
		if (self.keyMap["cam-left"]!=0):
			base.camera.setX(base.camera, +(elapsed*5.0))
		if (self.keyMap["cam-right"]!=0):
			base.camera.setX(base.camera, -(elapsed*5.0))
		
		# поворот камеры по вертикали
		if (self.keyMap["cam-up"]!=0):
			base.camera.setZ(base.camera, +(elapsed*5.0))
		if (self.keyMap["cam-down"]!=0):
			base.camera.setZ(base.camera, -(elapsed*5.0))
		
		# анимация выстрела
		if (self.keyMap["fire"]!=0):
			povorotS = self.stvol.getHpr()
			self.stvol.setHpr(0, 6, 0)
			self.keyMap["fire"]=0
		if (self.keyMap["fire"]==0):
			povorotS = self.stvol.getHpr()
			if povorotS[1]>0:
				self.stvol.setHpr(0, povorotS[1]-.5, 0)
		
		# поворот башни
		povorotB = self.bashnya.getHpr()
		if (self.keyMap["bashnya-left"]!=0):
			self.bashnya.setH(self.bashnya, +(elapsed*60.0))
		if (self.keyMap["bashnya-right"]!=0):
			self.bashnya.setH(self.bashnya, -(elapsed*60.0))
		
		# поворот танка
		if (self.keyMap["left"]!=0):
			self.a = (self.a + elapsed*75.0)%360
		if (self.keyMap["right"]!=0):
			self.a = (self.a - elapsed*75.0)%360
		
		# движеие танка
		if (self.keyMap["forward"]!=0):
			self.tank.setY(self.tank, elapsed*10000)
		if (self.keyMap["back"]!=0):
			self.tank.setY(self.tank, -(elapsed*5000))
			
		# возврат в исходное состояние камеры? и башни
		if (self.keyMap["norma"]!=0): 
			self.bashnya.setHpr(0, 0, 0)
		#####################################################################
		
		#
		startpos = self.tank.getPos()
		
		#
		self.cTrav.traverse(render)
		####################################################################################
		# вычисление высоты танка по поверхности 1
		entries = []
		for i in range(self.tankGroundHandler1.getNumEntries()):
			entry = self.tankGroundHandler1.getEntry(i)
			entries.append(entry)
		entries.sort(lambda x,y: cmp(y.getSurfacePoint(render).getZ(),x.getSurfacePoint(render).getZ()))
		if (len(entries)>0):
			# смягчаем прыжок по Z тут какраз и нужен тег который мы задали в начале
			Znew=entries[0].getSurfacePoint(render).getZ()
			Zold = float(self.tank.getTag('Zold'))
			deltaZ = (Znew - Zold)/1.20
			self.tank.setZ(Znew-deltaZ)
			
			if (Zold - Znew)>0.1:
				self.tank.setY(self.tank, elapsed*5000)
			
			if (Zold - Znew)>0.05:
				#self.tank.setY(self.tank, elapsed*5000)
				self.EnvColNp1.setZ(2000)
				self.EnvColNp2.setZ(2000)
				self.EnvColNp3.setZ(2000)
				self.EnvColNp4.setZ(2000)
			else:
				self.EnvColNp1.setZ(0)
				self.EnvColNp2.setZ(0)
				self.EnvColNp3.setZ(0)
				self.EnvColNp4.setZ(0)
			
			self.tank.setTag('Zold', str(self.tank.getZ()))
			#print Znew, Zold, deltaZ
		else:
			self.tank.setPos(startpos)
		
		# самое интересное попытка наклонять танк повторяя поверхность по которой он движется - в разработке
		if entries:
			
			#"""
			angleHold = float(self.tank.getTag('angleHold'))
			anglePold = float(self.tank.getTag('anglePold'))
			angleRold = float(self.tank.getTag('angleRold'))
			angleH=(180*entries[0].getInto().getNormal().getX())/3.1415
			angleP=(180*entries[0].getInto().getNormal().getY())/3.1415
			angleR=-(180*(entries[0].getInto().getNormal().getZ()-1))/3.1415
			deltaH = (angleHold - angleH)/10
			deltaP = (anglePold - angleP)/10
			deltaR = (angleRold - angleR)/10
			
			self.tank.setTag('angleHold', str(self.tank.getH()))
			self.tank.setTag('anglePold', str(self.tank.getP()))
			self.tank.setTag('angleRold', str(self.tank.getR()))
			
			#
			self.hprPole.setPos(entries[0].getSurfacePoint(render).getX(),entries[0].getSurfacePoint(render).getY(),entries[0].getSurfacePoint(render).getZ())
			self.hprPole.setHpr(angleH,-angleP,angleR)
			self.hprTank.setPos(self.tank.getPos())
			self.hprTank.setHpr(self.tank.getHpr())
			#
			
			n = entries[0].getInto().getNormal()
			q = Quat()
			q.setFromAxisAngle(self.a,n)
			self.tank.setQuat(q) 
			
			#
			ang=(self.hprTank.getHpr()-self.hprPole.getHpr())
			ttt=u"Полигон: H=%s, P=%s, R=%s\nТанк: H=%s, P=%s, R=%s\nРазница: %s\nq=%s"%(angleH,angleP,angleR,self.tank.getH(),self.tank.getP(),self.tank.getR(),ang,q)
			self.textObject.setText(ttt)
			
		#######################################################################
		
		# двигаем boxCam за танком
		self.boxCam.setPos(self.tank.getPos())
		self.boxCam.setZ(self.tank.getPos()[2]+1)
		#self.boxCam.setHpr(self.tank.getHpr())
		self.boxCam.setH(self.tank.getH())
		# двигаем камеру за танком точнее за водителем танка boxCam
		base.camera.reparentTo(self.boxCam)
		base.camera.lookAt(self.boxCam)
		#
		
		#выходим из обработчика
		return Task.cont
	
	# вывод инструкции
	def addInstructions(self, pos, msg):
		return OnscreenText(text=msg, style=1, fg=(1,1,1,1), font = self.font, pos=(-1.3, pos), align=TextNode.ALeft, scale = .04, shadow=(0,0,0,1))

	# вывод текста
	def addTitle(self, text):
		return OnscreenText(text=text, style=1, fg=(1,1,1,1), font = self.font, pos=(1.3,-0.95), align=TextNode.ARight, scale = .04, shadow=(0,0,100,1))

app = MyApp()
#render.explore()
app.run()
